Package: omicplotR
Title: Visual Exploration of Omic Datasets Using a Shiny App
Version: 1.32.0
Date: 2019-11-12
Authors@R: c(person("Daniel", "Giguere", email = "dgiguer@uwo.ca", role = c("aut", "cre")), person("Jean", "Macklaim", role = "aut"), person("Greg", "Gloor", role = "aut"))
biocViews: Software, DifferentialExpression, GeneExpression, GUI,
        RNASeq, DNASeq, Metagenomics, Transcriptomics, Bayesian,
        Microbiome, Visualization, Sequencing, ImmunoOncology
Description: A Shiny app for visual exploration of omic datasets as
    compositions, and differential abundance analysis using ALDEx2. Useful for
    exploring RNA-seq, meta-RNA-seq, 16s rRNA gene sequencing with
    visualizations such as principal component analysis biplots (coloured using
    metadata for visualizing each variable), dendrograms and stacked bar plots,
    and effect plots (ALDEx2). Input is a table of counts and metadata file (if
    metadata exists), with options to filter data by count or by metadata to
    remove low counts, or to visualize select samples according to selected
    metadata.
Imports: compositions, DT, grDevices, knitr, jsonlite, matrixStats,
        rmarkdown, shiny, stats, vegan, zCompositions
VignetteBuilder: knitr
Depends: R (>= 3.6), ALDEx2 (>= 1.18.0)
License: MIT + file LICENSE
LazyData: true
Encoding: UTF-8
RoxygenNote: 6.0.1
git_url: https://git.bioconductor.org/packages/omicplotR
git_branch: RELEASE_3_23
git_last_commit: 95307fd
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:39:26 UTC; biocbuild
Author: Daniel Giguere [aut, cre],
  Jean Macklaim [aut],
  Greg Gloor [aut]
Maintainer: Daniel Giguere <dgiguer@uwo.ca>
Built: R 4.6.1; ; 2026-08-05 18:27:55 UTC; unix
