exportPattern("^[^\\.]")

import("knitr")
import("rmarkdown")
importFrom("jsonlite", "fromJSON")
importFrom("stats", "dist")
importFrom("vegan", "anosim", "adonis")
importFrom("stats", "prcomp", "quantile", "dist")
importFrom("grDevices", "rgb", "col2rgb", "colorRampPalette")
importFrom("shiny", "validate", "need")
importFrom("DT", "renderDataTable", "dataTableOutput")
